<?php

/**
 * Gestion de l'action ordonnant les albums liés à un objet selon un ordre donné
 *
 * @note
 * Prévue pour un appel par un script JS effectuant une requête ajax en POST, avec un retour json.
 *
 * @note
 * On utilise volontairement des signatures génériques
 * en prévision d'une éventuelle intégration future dans le core,
 * ainsi à ce moment on n'aura pas à trop changer les appels.
 * Donc pas de table spip_albums_liens en dur, etc.
 *
 * @plugin     Albums
 * @copyright  2012-2022
 * @author     Tcharlss
 * @licence    GNU/GPL
 * @package    SPIP\Albums\Action
 */

// Sécurité
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

/**
 * Ordonner les albums liés à un objet selon un ordre donné
 *
 * @note
 * On ne passe pas pas sécuriser_action(), celle-ci est appelée par un script JS anonyme.
 * On vérifie toutefois l'autorisation de modifier l'objet lié.
 *
 * @return string
 *    Objet json avec 3 clés `done`, `success` et `errors`
 */
function action_ordonner_liens_albums_dist() {
	include_spip('inc/autoriser');
	include_spip('base/objets');
	include_spip('action/editer_liens');

	// Objet source possédant la table de liens  et éventuellement la colonne portant le rang
	// Si pas donné, on prend en dur pour l'instant. Pourrait-être donné plus tard par objet_associable() ?
	$objet = objet_type(_request('objet_source'));
	$champ_rang = _request('champ') ?: 'rang_lien';

	// Objet lié
	$objet_lie = objet_type(_request('objet_lie'));
	$id_objet_lie = intval(_request('id_objet_lie'));

	// Ordre des éléments (tableau)
	$ordre = _request('ordre');

	// Vérifs : il faut avoir le droit de modifier l'objet lié
	if (
		!$objet
		or !$objet_lie
		or !$id_objet_lie
		or !$ordre
		or !is_array($ordre)
		or !objet_associable($objet)
	) {
		return envoyer_json_erreur(_T('medias:erreur_objet_absent') . ' ' . _T('medias:erreur_deplacement_impossible'));
	} elseif (!autoriser('modifier', $objet_lie, $id_objet_lie)) {
		return envoyer_json_erreur(_T('medias:erreur_autorisation') . ' ' . _T('medias:erreur_deplacement_impossible'));
	}

	[$cle_objet, $table_liens] = objet_associable($objet);

	$success = $errors = [];

	$actuels = sql_allfetsel(
		[$cle_objet . ' AS id', $champ_rang],
		$table_liens,
		[
			sql_in($cle_objet, $ordre),
			'objet = ' . sql_quote($objet_lie),
			'id_objet = ' . sql_quote($id_objet_lie)
		]
	);

	$futurs = array_flip($ordre);
	// ordre de 1 à n (pas de 0 à n).
	array_walk($futurs, function (&$v) {
		$v++;
	});

	$updates = [];

	foreach ($actuels as $l) {
		if ($futurs[$l['id']] !== $l[$champ_rang]) {
			$updates[$l['id']] = $futurs[$l['id']];
		}
	}

	if ($updates) {
		foreach ($updates as $id => $ordre) {
			sql_updateq(
				$table_liens,
				[$champ_rang => $ordre],
				[
					$cle_objet . ' = ' . $id,
					'objet = ' . sql_quote($objet_lie),
					'id_objet = ' . sql_quote($id_objet_lie)
				]
			);
		}
	}

	return ordonner_liens_albums_json_envoi([
		'done' => true,
		'success' => $success,
		'errors' => $errors,
	]);
}

function ordonner_liens_albums_json_envoi($data) {
	header('Content-Type: application/json; charset=' . $GLOBALS['meta']['charset']);
	echo json_encode($data, JSON_THROW_ON_ERROR);
}

function ordonner_liens_albums_json_erreur($msg) {
	return ordonner_liens_albums_json_envoi([
		'done' => false,
		'success' => [],
		'errors' => [$msg]
	]);
}
